/**
* @license
* Copyright © 2015 Intuilab
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
* to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
* and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
* 
* The Software is provided "as is", without warranty of any kind, express or implied, including but not limited to the warranties of merchantability, 
* fitness for a particular purpose and noninfringement. In no event shall the authors or copyright holders be liable for any claim, damages or other liability, 
* whether in an action of contract, tort or otherwise, arising from, out of or in connection with the Software or the use or other dealings in the Software.
* 
* Except as contained in this notice, the name of Intuilab shall not be used in advertising or otherwise to promote the sale, 
* use or other dealings in this Software without prior written authorization from Intuilab.
*/

/**
* Inheritance on EventEmitter base class
* @type {EventEmitter}
*/
ValueToColorsBC.prototype = new Object();          // Here's where the inheritance occurs
ValueToColorsBC.prototype.constructor = ValueToColorsBC;

/**
* @constructor
*/
function ValueToColorsBC() {

}


/**
 * @method
 */
ValueToColorsBC.prototype.valueToColors = function (input, inputMin, start_color, inputMax, end_color) {

    try{

        if ((input == null) || isNaN(input) || (inputMin == null) || isNaN(inputMin) || (inputMax == null) || isNaN(inputMax)) {
            return null;
        }
        else if (start_color == null || end_color == null) {
            return null;
        }
        else{
            var percent;

            if ((parseFloat(inputMax) != parseFloat(inputMin))){
                percent = (parseFloat(input) - parseFloat(inputMin)) / (parseFloat(inputMax) - parseFloat(inputMin));
            }
            else{
                percent = 0.5;
            }

            if (percent < 0) {
                percent = 0;
            }
            else if (percent > 1) {
                percent = 1;
            }

            // strip the leading # if it's there
            start_color = start_color.replace(/^\s*#|\s*$/g, '');
            end_color = end_color.replace(/^\s*#|\s*$/g, '');

            // get alpha and colors value
            var start_alpha = parseInt(start_color.substr(0, 2), 16),
                start_red = parseInt(start_color.substr(2, 2), 16),
                start_green = parseInt(start_color.substr(4, 2), 16),
                start_blue = parseInt(start_color.substr(6, 2), 16);

            var end_alpha = parseInt(end_color.substr(0, 2), 16),
                end_red = parseInt(end_color.substr(2, 2), 16),
                end_green = parseInt(end_color.substr(4, 2), 16),
                end_blue = parseInt(end_color.substr(6, 2), 16);

            // calculate diff between start and end colors
            var diff_alpha = end_alpha - start_alpha;
            var diff_red = end_red - start_red;
            var diff_green = end_green - start_green;
            var diff_blue = end_blue - start_blue;

            // calculate new color result
            var result_alpha = ( (diff_alpha * percent) + start_alpha ).toString(16).split('.')[0];
            var result_red = ( (diff_red * percent) + start_red ).toString(16).split('.')[0];
            var result_green = ( (diff_green * percent) + start_green ).toString(16).split('.')[0];
            var result_blue = ( (diff_blue * percent) + start_blue ).toString(16).split('.')[0];

            // ensure 2 digits by alpha
            if( result_alpha.length == 1 )
                result_alpha = '0' + result_alpha;

            // ensure 2 digits by color
            if( result_red.length == 1 )
                result_red = '0' + result_red;
            if( result_green.length == 1 )
                result_green = '0' + result_green;
            if( result_blue.length == 1 )
                result_blue = '0' + result_blue;

            // return the Hex color result, with #
            return '#' + result_alpha + result_red + result_green + result_blue;

        }
    }
    catch (e){
        return null;
    }


};




ValueToColorsBC.prototype._DecToHexa = function (DecNb) {
    var hexa = "0123465789ABCDEF";
    var x = Math.floor(DecNb / 16);
    var h = hexa.charAt(x);
    x = DecNb % 16;
    h += hexa.charAt(x);

    return h;
};

ValueToColorsBC.prototype._Degrade = function (dr,dg,db,fr,fg,fb,texte) {
    var steps = texte.length;
    var cr = dr;
    var cg = dg;
    var cb = db;

    var sr = (fr - dr) / steps;
    var sg = (fg - dg) / steps;
    var sb = (fb - db) / steps;

    for (var x = 0; x <= steps; x++) {
        console.log("Color:" + this._DecToHexa(cr) + this.DecToHexa(cg) + this.DecToHexa(cb));
        cr += sr; cg += sg; cb += sb;
    }
};



//Convert a number to a hexadecimal string with:
//hexString = yourNumber.toString(16);

//and reverse the process with:
//yourNumber = parseInt(hexString, 16);

